// ============================================================================
// Project:	Deductive Database
// Filename:	in_file.h
// Purpose:	Get Input From a File
// Last Change:	26.08.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 1998-2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	IN_FILE_INCLUDED
#define IN_FILE_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifdef VER_IOSTREAM
#include <iostream>
#include <fstream>
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef IN_INCLUDED
#include "in.h"
#endif

//=============================================================================
// Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// IN_FILE_NAMESIZE: Maximal filename length.
//-----------------------------------------------------------------------------

static const int IN_FILE_NAMESIZE = 128;

//-----------------------------------------------------------------------------
// IN_FILE_CGI_FILENAME_CHARS: Valid characters in filenames read via web.
//-----------------------------------------------------------------------------

static const str_t IN_FILE_CGI_FILENAME_CHARS =
	"_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
	// Make sure that / or \ are not part of this string (for security).
	// In addition, the program permits the extension ".txt".

//=============================================================================
// Class for Input File:
//=============================================================================

class in_file_c : public in_c {
	public:

//-----------------------------------------------------------------------------
// Constructor:
//-----------------------------------------------------------------------------

	in_file_c(str_t filename);

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

	~in_file_c();

//-----------------------------------------------------------------------------
// close: Close input file explicitly.
//-----------------------------------------------------------------------------

	bool	close();

//-----------------------------------------------------------------------------
// read: Read characters from input file into buffer.
//-----------------------------------------------------------------------------

	int	read(char *buf, int size);

//-----------------------------------------------------------------------------
// is_open: Is this input stream currently open?
//-----------------------------------------------------------------------------

	inline bool	is_open()
#ifdef VER_IOSTREAM
		{ return(File.is_open()); }
#else
		{ return(Fd != -1); }
#endif

//-----------------------------------------------------------------------------
// filename: Name of this file.
//-----------------------------------------------------------------------------

	inline str_t	filename()
		{ return(Filename); }

//=============================================================================
// Private Class Members:
//=============================================================================

	private:

//-----------------------------------------------------------------------------
// Fd: The file descriptor of the input file.
//-----------------------------------------------------------------------------

#ifdef VER_IOSTREAM
	std::ifstream File;
#else
	int	Fd;
#endif

//-----------------------------------------------------------------------------
// Filename: The name of the input file.
//-----------------------------------------------------------------------------

	char	Filename[IN_FILE_NAMESIZE+1];

//=============================================================================
// End of Class:
//=============================================================================

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef in_file_c *in_file_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define IN_FILE_NULL (static_cast<in_file_t>(0))

//=============================================================================
// End of File:
//=============================================================================

#endif

